package tumbler;

import static tumbler.internal.TumblerUtils.*;

import java.util.*;

import org.junit.runner.*;
import org.junit.runner.notification.*;
import org.junit.runners.*;
import org.junit.runners.model.*;

import tumbler.internal.*;

/**
 * JUnit runner. Represents stories and scenarios as sentences in JUnit GUIs.
 * May also generate scenarios' statuses report in .scenarios or html format if
 * <code>-DgenerateReport=[scenario|html]</code> is passed while running
 * scenarios. The output folder for the report can be configured by setting
 * <code>-DoutputFolder</code> property.
 * 
 * If -DcamelCase is passed, IDEs will show class and method names instead of
 * scenario and story names.
 * 
 * @author Pawel Lipinski
 */
public class TumblerRunner extends BlockJUnit4ClassRunner {

	private ScenarioListener scenarioListener;
	private Result result;
	private RunListener runListener;

	public TumblerRunner(Class<?> klass) throws InitializationError {
		super(klass);
	}

	@Override
	public void run(RunNotifier notifier) {
		setupScenariosListener(notifier);
		if (getTestClass().getJavaClass().getPackage() != null)
			scenarioListener.getStory().withPackage(getTestClass().getJavaClass().getPackage().getName());
		super.run(notifier);
		removeScenariosListener(notifier);
	}

	@Override
	protected void runChild(FrameworkMethod method, RunNotifier notifier) {
		super.runChild(method, notifier);
		if (method.getAnnotation(Scenario.class).pending())
			notifier.fireTestIgnored(describeChild(method));
	};

	private void removeScenariosListener(RunNotifier notifier) {
		notifier.fireTestRunFinished(result);

		notifier.removeListener(scenarioListener);
		notifier.removeListener(runListener);
	}

	private void setupScenariosListener(RunNotifier notifier) {
		result = new Result();
		runListener = result.createListener();
		notifier.addFirstListener(runListener);

		scenarioListener = new ScenarioListener();
		notifier.addListener(scenarioListener);
		notifier.fireTestRunStarted(getDescription());
	};

	@Override
	protected java.util.List<FrameworkMethod> computeTestMethods() {
		List<FrameworkMethod> testMethods = super.computeTestMethods();
		if (! testMethods.isEmpty())
			throw new RuntimeException("Class " + super.getName() + " is a Tumbler story. It should not use @Test annotation (but it did!) \n Remove @Test annotation from: " + testMethods);
		
		return getTestClass().getAnnotatedMethods(Scenario.class);
	};

	@Override
	protected String getName() {
		if (shouldSkipNameResolution())
			return super.getName();

		String name = super.getName();
		name = name.substring(name.lastIndexOf('.') + 1);
		if (name.endsWith("Test"))
			name = name.substring(0, name.lastIndexOf("Test"));
		if (name.endsWith("Scenarios"))
			name = name.substring(0, name.lastIndexOf("Scenarios"));
		return decamelise(name) + " should";
	};

	@Override
	protected String testName(FrameworkMethod method) {
		if (shouldSkipNameResolution())
			return super.testName(method);

		String scenarioName = decamelise(super.testName(method));
		String lowerCaseScenario = scenarioName.toLowerCase();
		if (lowerCaseScenario.startsWith("should") || lowerCaseScenario.startsWith("test"))
			scenarioName = skipFirstWord(scenarioName);
		return scenarioName;
	}

	private boolean shouldSkipNameResolution() {
		return System.getProperty("camelCase") != null;
	}

	private String skipFirstWord(String sentence) {
		return sentence.substring(sentence.indexOf(" "));
	}
}
