package tumbler.internal.domain;

import java.io.*;
import java.util.*;

import org.junit.runner.*;

import tumbler.internal.*;
import tumbler.internal.domain.ScenarioModel.*;
import tumbler.internal.parsers.*;

public class StoryModel {

	private String name;
	private List<ScenarioModel> scenarios = new ArrayList<ScenarioModel>();
	private String packageName = "";

	public StoryModel() {
	}

	public StoryModel(String name) {
		this.name = name.trim();
	}

	public String name() {
		return name;
	}

	public StoryModel withPackage(String packageName) {
		this.packageName = packageName;
		return this;
	}

	public List<ScenarioModel> scenarios() {
		return Collections.unmodifiableList(scenarios);
	}

	public String asClassPath() {
		if (!"".equals(packageName))
			return packageName.replace('.', '/') + "/" + camelisedName();
		else
			return camelisedName();
	}

	public void addScenario(ScenarioModel scenario) {
		if (scenarios.contains(scenario))
			throw new ParseException("Two scenarios with the same name: " + scenario.name());

		scenarios.add(scenario);
		scenario.setStory(this);
	}

	public ScenarioStatus status() {
		for (ScenarioModel scenario : scenarios) {
			if (scenario.status() != ScenarioStatus.PASSED)
				return scenario.status();
		}
		return ScenarioStatus.PASSED;
	}

	public static StoryModel createFrom(String firstLine) throws IOException {
		StoryModel story = new StoryModel();
		story.parseNameFrom(firstLine);
		return story;
	}

	private void parseNameFrom(String firstLine) throws IOException {
		checkStoryExistence(firstLine);
		String storyName = retrieveStoryFromLine(firstLine);
		checkIfStoryNotEmpty(storyName);
		name = storyName;
	}

	private void checkIfStoryNotEmpty(String storyName) {
		if (storyName.isEmpty())
			throw new IllegalArgumentException("Story is empty");
	}

	private String retrieveStoryFromLine(String firstLine) {
		return firstLine.trim().substring(firstLine.indexOf(':') + 1).trim();
	}

	private void checkStoryExistence(String firstLine) {
		if (firstLine == null 
				|| firstLine.trim().isEmpty() 
				|| ! (
					firstLine.trim().startsWith("Story:") || firstLine.trim().startsWith("Feature:")
				))
			throw new IllegalArgumentException("Scenarios have no story");
	}

	public String camelisedName() {
		String result = TumblerUtils.removeSpecialCharacters(name);
		return TumblerUtils.camelise(result);
	}

	@Override
	public boolean equals(Object obj) {
		StoryModel other = (StoryModel) obj;
		if (namesNotSame(other))
			return false;
		return (other.scenarios.containsAll(scenarios) && scenarios.containsAll(other.scenarios));
	}

	private boolean namesNotSame(StoryModel other) {
		return !((name == null && other.name == null) || name.equals(other.name));
	}

	@Override
	public String toString() {
		return "Story: " + name + "; Scenarios:" + scenarios;
	}

	public ScenarioModel scenarioDescribedBy(Description description) {
		for (ScenarioModel scenario : scenarios)
			if (scenario.description() == description)
				return scenario;
		return null;
	}

	public RunStatistics runStatistics() {
		int[] stats = new int[] { 0, 0, 0 };
		for (ScenarioModel scenario : scenarios)
			if (scenario.status() != null)
				stats[scenario.status().ordinal()]++;

		return RunStatistics.fromArray(stats);
	}

	public String packageName() {
		return packageName;
	}
}
